/*******************************************************************************
Copyright (c) 2000 Oracle Corporation

File        : form.js

Notes       : Some functions can only used in UIX pages others only in JSP pages
              and some in both. This is indicated by a note, e.g. note: UIX & JSP

Open Issues : functions that are not called by any generated coded (directly or
              indirectly) should be refactored out and put into a specific reuse
              library.

$revision_history$
24-mei-2005   Steven Davelaar
  1.20        - PEBELL: Added resetBreadcrumbs param to doNavigate
12-mei-2005   Aard-Jan van Kesteren
  1.19        In validateForm check whether a table actually exists before validating it
09-mei-2005   Aard-Jan van Kesteren
  1.18        If no hidden field validateForm is present, always disable validation instead
              of a javascript error
29-apr-2005   Aard-Jan van Kesteren
  1.17        - Added clearAdvancedSearch
              - Replaced item[0].name checks with item[0] && item[0].name to make javascript more robust
29-apr-2005   Steven Davelaar
  1.15        Added method selectTreeNode
28-apr-2005   Aard-Jan van Kesteren
  1.14        Correct handling of array items in the validateForm mechanism
27-apr-2005   Aard-Jan van Kesteren
  1.13        Better handling of array items in getItemType
21-apr-2005   Aard-Jan van Kesteren
  1.12        Moved formValidationEnabled logic from checkForChanges to validateForm
21-apr-2005   Aard-Jan van Kesteren
  1.11        - Added addToItemScreenPrompts and addToRequiredRowItems
              - Added validateForm
              - Added tablesToBeValidated
              - Refactored checkForChanges such that it doesn't need the changes hidden
                field anymore
14-apr-2005   Steven Davelaar
  1.10        - added methods enableFormValidation, disableFormValidation, 
                and isFormValidationDisabled
              - Changed LovSelectCallBack and checkForChanges to use above
                methods
05-feb-2005   Steven Davelaar
  1.8         Improved check in ignoreField
03-feb-2005   Jaco Verheul
  1.7         Expanded width of jsp lov window
07-jan-2005   JACO
  1.6         Checked in after upload from K:\adf\runtime\web\jheadstart\form.jsRemoved unused function toURLString. Function caused Unicode problems
16-nov-2004   Peter Ebell
  1.5         Fix for JDev10.1.2: error in StripRownumFromItemName when clicking UIX tab or button from table page
11-nov-2004   Sandra Muller
  1.4         changed %D1 and %D2 lines in URLtoString (needed for Japanese)
30-jul-2004   Steven Davelaar
  1.3         Added function LovSelectCallBack, alertForChanges
04-jun-2004   Steven Davelaar
  1.2         Removed some obsolete functions
04-jun-2004   Steven Davelaar
  1.1         Added submitJspAdfForm
10-feb-2001   Steven Davelaar
  1.0         Copied from form.js of JHeadstart 9.0.4.5
******************************************************************************/
/*
$add_to_revision_history
$
*/

/* Debug procedure */
var debugMode = false;

/* View type. The default is UIX, in generated JSP pages it will be set to JSP */
var viewType="UIX";

/**
 * debug gives debug messages based on the setting of debugMode
 * the debugMode can be switched of during debugging
 */
function debug(text)
{
  if (debugMode)
  {
    debugMode = confirm(text);
  }
} // debug


// Global variables.
/**
 * form contains the name of the form that should be submitted.
 */
var form;

/**
 * field contains the name of the field that should get some value.
 */
var field;

/**
 * reset contains an array of fields that should be reset before a form
 * submission.
 */
var reset;

/**
 * if ignoreChanges has a value then the appropriate functions don't check
 * for changes.
 */
var ignoreChanges;

/**
 * ignoreChangedFields is an array that contains form
 * fields that are ignored while checking a page for changes.
 */
var ignoreChangedFields = new Array();

/**
 * requiredFields contains the names of the items that are required in a marlin table.
 * Required fields cannot be enforced by marlin validation if empty rows are used.
 * The fields are contained, without the table name, for example:
 *   ['fields', 'fields']
 * This array has been depricated and is replaced by the array requiredTableItems
 */
var requiredFields = new Array();

/**
 * requiredRowItems contains the names of the items that are required in a marlin table.
 * Required table items cannot be enforced by marlin validation if empty rows are used.
 * The items are contained, including the table name, for example:
 *   ['table0:item0','table0:item1','table1:item0']
 */
var requiredRowItems = new Array();

/**
 * itemScreenPrompts contains the prompts of all form items.
 */
var itemScreenPrompts = new Array();

/**
 * tablesToBeValidated contains all the table names
 * that need to be validated on submit. 
 */
var tablesToBeValidated = new Array();

/**
 * These are public variables that are used by CheckForChanges, and its helper functions
 */
var dataForm;


/**
 * JSP specific variables
 */
var lovWindow;
var calendarWindow;
var editorWindow;
var lovField;
var baseValueAttribute;
var editorValue;
var editorField;
var editorMaxlength;


/**
 * getItemName gets the 'technical' name of an item. In case of UIX table item names have
 * the format [tableName]:[itemName]:[rownum], in which case the rownum needs to be stripped
 * note: UIX & JSP
 * @param item: item to get the name for
 * @return: the name of the item
 */
function getItemName(item)
{
  var type;
  var itemName;
  var colon1;
  var colon2;

  debug("in getItemName");
  if (!item)
  {
    alert("undefined item >"+item+"< in getItemName");
    return null;
  }
  else
  {
    type = getItemType(item);
    if (type == "radiogroup")
    {
      itemName = item[0].name;
    }
    else if (!item.name && item[0] && item[0].name)
    {
      // Apparently item is an array. Just take the name of the first item
      itemName = item[0].name;
    }     
    else
    {
      itemName = item.name;
    }

    // only in case of UIX does the item name contain the rownum
    if (viewType == "UIX")
    {
      itemName = stripRownumFromName(itemName);
    }
  }
  return itemName;
} // getItemName


/**
 * constructTableItemName construct the HTML item name of an item in a table. In case of UIX
 * table item names have the format [tableName]:[itemName]:[rownum]. In case of JSP,
 * table item names have the format [itemName]:[[rownum]]
 * note: UIX & JSP
 * @param itemName: the item name including the UIX table name, when applicable
 * @param rownum: the index of the item in the table
 * @return: the name of the item as it will be rendered in the HTML table
 */
function constructTableItemName(itemName, rownum)
{
  var tableItemName;

  if (!itemName)
  {
    alert("undefined itemName in constructTableItemName");
    return null;
  }
  if (!rownum)
  {
    alert("undefined rownum in constructTableItemName");
    return null;
  }

  if (viewType == "UIX")
  {
    tableItemName = itemName+":"+rownum;
  }
  else
  {
    tableItemName = itemName+"["+rownum+"]";
  }
  return tableItemName;
} // constructTableItemName


/**
 * stripTableItemName strips the rowNum from items that have the format
 * [tableName]:[itemName]:[rownum]
 * note: UIX
 * @param itemName: the itemName before stripping the rownum
 * @return: the 'short' name of the item
 */
function stripRownumFromName(itemName)
{
  colon1 = itemName.indexOf(":");
  colon2 = itemName.lastIndexOf(":");
  if (colon1 < colon2)
  { // strip the rownum from the itemName
    itemName = itemName.substring(0,colon2);
  }

  return itemName;
} // stripRownumFromTableItemName


/**
 * hasChanged checks if the value of item has changed. Thereby also hidden items are checked.
 * note: UIX & JSP
 * @param item: the item to check
 * @return: true when the item has changed, false otherwise
 */
function hasChanged(item)
{
  // debug("In hasChanged for item: "+item);
  var changed = false;

  if (!item)
  {
    alert("undefined item >"+item+"< in hasChanged");
    changed = null;
  }
  else
  {
    // getItemType will discover when item is an invalid parameter
    var type = getItemType(item);
    var itemName = getItemName(item);

    if (!ignoreField(itemName))
    {
      var value;
      var defaultValue;
      var checked;
      var defaultChecked;
      // Check if item is an array. If so, handle differently than if it's
      // a single object
      if (!item.name && item[0] && item[0].name)
      {
        defaultValue = item[0].defaultValue;
        value = item[0].value;
        checked = item[0].checked;
        defaultChecked = item[0].defaultChecked;
      }
      else
      {
        defaultValue = item.defaultValue;
        value = item.value;
        checked = item.checked;
        defaultChecked = item.defaultChecked;
      }    
    
      if (  (  (type == "password" || type == "text" || type == "textarea")
            && defaultValue != value
            )
         || (  (type == "checkbox" || type == "radio")
            && defaultChecked != checked
            )
         )
      {
        changed = true;
      }
      else if (type == "radiogroup")
      {
        for (var i = 0; i < item.length && !changed; i++)
        {
          changed = (item[i].defaultChecked != item[i].checked);
        }
      }
      else if (type == "select-one" || type == "select-multiple")
      {
        var options         = item.options;
        var defaultSelected = 0;
        for (var j = 0; j < options.length; j++)
        {
          if (options[j].defaultSelected)
          {
            defaultSelected = j;
          }
        }

        if (  (defaultSelected != options.selectedIndex)
           && (options.selectedIndex != -1)
           )
        {
          changed = true;
        }
      }
      else if (type == "hidden")
      {
        changed = (item.value != item.defaultValue);
      }
    }
  }

  return changed;
} // hasChanged


/**
 * isEmptyDisplayItem checks if the item is "empty" from the end user perspective.
 * note: UIX & JSP
 * @param item: the item to check
 * @return: true when it is an empty display item, false otherwise
 */
function isEmptyDisplayItem(item)
{
  var empty = true;
  var type;

  if (!item)
  {
    alert("undefined item >"+item+"< isEmptyDisplayItem");
    return null;
  }
  else
  {
    // getItemType will discover when item is an invalid parameter
    var type  = getItemType(item);
    if (type != "hidden")
    {
      empty = isEmptyItem(item);
    }
    else // hidden items are never an empty display item
    {
      empty = false;
    }
    debug("item "+item.name+": empty is " + empty);
    return empty;
  }
} // isEmptyDisplayItem


/**
 * Get the value of the "hasChanges" item of a specific row in the table
 * Helper function for checkForChanges.
 * note: UIX
 * @param tableName: the name of the table
 * @param rownum: the number of the row
 * @return: the value of hasChanges
 */
function getChangedStatus(tableName, rownum)
{
  return dataForm[tableName+":hasChanges:"+rownum].value;
} // getChangedStatus


/**
 * setChangedStatus sets the value of the "hasChanges" item of a specific row in the table
 * Helper function for checkForChanges.
 * note: UIX
 * @param tableName: the name of the table
 * @param rownum: the number of the row
 * @param value: the value to set
 */
function setChangedStatus(tableName, rownum, value)
{
  dataForm[tableName+":hasChanges:"+rownum].value = value;
} // setChangedStatus


/**
 * userHasChanged checks if an item has been changed by the user. Therefore only non-hidden
 * items are checked.
 * note: UIX & JSP
 * @param item: the item to check
 * @return: true when the value has changed, false otherwise
 */
function userHasChanged(item)
{
  if (!item)
  {
    alert("undefined item >"+item+"< in userHasChanged");
    return null;
  }
  else
  {
    var type = getItemType(item);
    // only displayed items can return true
    if (type != "hidden")
    {
      return hasChanged(item);
    }
    else
    {
      return false;
    }
  }
} // userHasChanged


/**
 * hasInsertStatus gets the value of the "inserting" field of a specific row in the table
 * Helper function for checkForChanges.
 * note: UIX
 * @param tableName: the name of the table
 * @param rownum: the number of the row
 * @return: true when inserting equals Y, false otherwise
 */
function hasInsertStatus(tableName, rownum)
{
  if (dataForm[tableName+":rowKeyStr:"+rownum])
  {
    debug("Row "+rownum+" inserting = "+dataForm[tableName+":rowKeyStr:"+rownum].value);
    return (dataForm[tableName+":rowKeyStr:"+rownum].value == '');
  }
  else
  {
    return false;
  }
} // hasInsertStatus


/**
 * isEmptyDisplayLine checks if all the visible items in a row are empty
 * Helper function for checkForChanges
 * note: UIX & JSP
 * @param formName: the name of the form
 * @param tableItems: array of table item names
 * @param rownum: the number of the row
 * @return: true when all visible items are empty, false otherwise
 */
function isEmptyDisplayLine(formName, tableItems, rownum)
{
  debug("in isEmptyDisplayLine");

  var form      = document.forms[formName];
  var emptyline = true;
  var type;
  var item;
  var tableItemName;

  for (var k = 0; (k < tableItems.length) && emptyline; k++)
  {
    // pass rownum as a string
    tableItemName = constructTableItemName(tableItems[k], rownum+"");
    item = form[tableItemName];
    // getItemType deals with undefined items
    type = getItemType(item);
    // ignore hidden items, checkboxes and radio buttons
    // checking for hidden & using isEmptyItem, because isEmptyDisplayItem
    // will return false for hidden items.
    if (  type != "hidden"
       && type != "checkbox"
       && type != "radio"
       && !isEmptyItem(item)
       )
    {
      emptyline = false;
    }
  }

  debug("Row "+ rownum+": empty line is "+ emptyline);
  return emptyline;
} // isEmptyDisplayLine


/**
 * createTableItemNamesArray creates an array of table item names
 * note: UIX
 * @param formName: the name of the form
 * @param tableName: the name of the table for which the array should be created
 * @return: an array of table item names ("table name:item name")
 */
function createTableItemNamesArray(formName, tableName)
{
  var form               = document.forms[formName];
  var formElementsCount  = form.elements.length / 1;
  var tempTableItemNames = new Array(formElementsCount);
  var itemFound          = new Array();
  var itemCount          = 0;

  var item;
  var itemName;
  var colon1;
  var colon2;

  // loop through all items in the form and add those items to tempTableItems
  // that belong to tableName
  for (var i = 0 ; i < formElementsCount ; i++)
  {
    item     = form.elements[i];
    itemName = new String(item.name);
    colon1   = itemName.indexOf(":");
    colon2   = itemName.lastIndexOf(":");
    // add item to the array when it is in table with tableName
    if (  colon1 > -1 && colon2 > -1 && colon1 != colon2
       && tableName == itemName.substring(0, colon1)
       )
    {
      // itemFound indicates if a specific item name already has been found
      if (!itemFound[itemName.substring(colon1+1,colon2)])
      {
        itemFound[itemName.substring(colon1+1,colon2)] = true;
        tempTableItemNames[itemCount] = itemName.substring(0,colon2);
        itemCount++;
      }
    }
  }

  // As there may be items in the form than do not belong to the table, the length
  // of tempTableItems may be longer than necessary. Therefore an array is created
  // with a length equal to the number of table items, to be returned.
  var tableItemNames = new Array(itemCount);
  for (var i = 0; i < itemCount; i++)
  {
    tableItemNames[i] = tempTableItemNames[i];
  }

  //  debugString = "Found the following items for table "+tableName+": \n\n";
  //  for (var i = 0 ; i < itemCount; i++) debugString +="- "+tableItemNames[i]+"\n";
  //  debug(debugString);

  return tableItemNames;

} // createTableItemNamesArray


/**
 * getScreenPrompt translates an item name to a prompt as displayed on the screen.
 * It is used for form layout and table layout, and also in samepage master-detail
 * pages so for items that are in a table, the table name needs to be added, e.g.:
 *   getScreenPrompt("Ename");
 *   getScreenPrompt("EmpDepSet:Ename");
 * note: UIX & JSP
 * @param itemName: the name of the item for which to get the prompt
 * @return: the prompt of the item if it could be found in itemScreenPrompts, the
 * name of the item otherwise
 */
function getScreenPrompt(itemName)
{
  var prompt;

  debug("in getScreenPrompt");
  if (!itemName)
  {
    alert("undefined itemName in getScreenPrompt");
    return null;
  }
  else
  {
    // For backward compatibility, check if itemScreenPrompts array exists.
    if (itemScreenPrompts)
    {
      for (var i = 0; i < itemScreenPrompts.length; i++)
      {
        if (itemScreenPrompts[i][0] == itemName)
        {
          debug("prompt for item "+itemName+" = "+itemScreenPrompts[i][1]);
          prompt = itemScreenPrompts[i][1];
        }
      }
    }
    if (!prompt) // itemScreenPrompts does not exists, or does not include itemName
    {
      prompt = itemName;
      // if itemName includes the table name remove the table name
      // in case of JSP we cannot strip the FormBean name
      var j  = prompt.lastIndexOf(':');
      if (j > -1)
      {
        prompt = itemName.substring(j+1);
      }
      debug("prompt for item is "+prompt+", because no prompt could be found");
    }
  }
  return prompt;
} // getScreenPrompt



/**
 * checkForEmptyRequiredRowItems checks for all required items in a row if they have
 * a value. An alert is given when a required item has no value.
 * note: UIX & JSP
 * @param formName: the name of the form
 * @param tableName: the table name
 * @param rownum: the index of the row to be checked
 * @return: true when all required items have a value, false otherwise
 */
function checkForEmptyRequiredRowItems(formName, rownum, tableName)
{
  // TO DO: should refactor this function as to interpret tableName as
  // FormBean name in case of JSP
  var form = document.forms[formName];
  var formItem;
  var type;
  var formItemName;
  var formTableItemName;

  debug("In checkForEmptyRequiredRowItems");

  // only check when requiredRowItems have been specified
  if (requiredRowItems)
  {
    // loop through all the required items
    for (var i = 0; i < requiredRowItems.length; i++)
    {
      debug("checking item "+i);
      // only check the items that have a name that includes the tableName
      // when no tableName has been passed it does not concern a UIX page
      if (  (  tableName
            && requiredRowItems[i].indexOf(tableName) > -1
            )
         || !tableName
         )
      {
        formItemName = requiredRowItems[i];
        // convert rownum to string
        formTableItemName = constructTableItemName(formItemName, rownum+"");
        formItem = form[formTableItemName];

        // non-updateble items are rendered as text, so it should be checked if the item
        // exists before checking the requiredness
        if (formItem)
        {
          debug("Checking for requiredness of item "+i+": "+formTableItemName+", formitem="+formItem);
          if (isEmptyDisplayItem(formItem))
          {
            // Show the message to fill in this field
            alert(getMessage("requiredFields")+getScreenPrompt(formItemName));
            type = getItemType(formItem);
            // Give the offending item the focus
            if (type == "radiogroup")
            {
              formItem[0].focus();
            }
            else
            {
              formItem.focus();
            }
            // return false as soon as an empty required item has been found
            debug("returning false in checkForEmptyRequiredRowItems because of item "+formItemName+":"+rownum);
            return false;
          }
        }
      }
    }
    // return true when no empty required items could be found
    return true;
  }
  else
  {
    // return true when there are no requiredRowItems
    return true;
  }
} // checkForEmptyRequiredRowItems


/**
 * checkRequiredRowItems is a function router that either calls
 * checkForEmptyRequiredRowFields() or checkForEmptyRequiredRowItems() for backward
 * compatability. JHeadstart for JDeveloper 9.0.2 and 9.0.3 included the array
 * requiredFields that contains item names without the table prefix. From 9.0.4 the
 * new array requiredRowItems do include the table name.
 * note: UIX & JSP
 * @param formName: the name of the form
 * @param tableName: the table name
 * @param rownum: the index of the row to be checked
 * @return: true when all required items have a value, false otherwise
 */
function checkRequiredRowItems(formName, tableName, rownum)
{
  // TO DO: should refactor this as to interpret tableName as FormBean name in case of JSP
  debug("start checkRequiredRowItems");
  var returnValue = true;

  if (  requiredFields
     && requiredFields.length > 0
     )
  { // JHeadstart 9.0.2 & 9.0.3
    debug("checkRequiredRowItems for jhs 902/903");
    returnValue = checkForEmptyRequiredRowFields(formName, rownum, tableName);
  }
  else if (  requiredRowItems
          && requiredRowItems.length > 0
          )
  { // JHeadstart 9.0.4
    debug("checkRequiredRowItems for jhs 904");
    returnValue = checkForEmptyRequiredRowItems(formName, rownum, tableName);
  }

  debug("returning "+returnValue+" in checkRequiredRowItems");
  return returnValue;
} // checkRequiredRowItems


/**
 * deleteRowChecked checks if the delete checkbox has been checked for a row
 * Helper function for checkForChanges.
 * note: UIX
 * @param tableName: the name of the table
 * @param rownum: the number of the row in the table
 * @return: true when deleteRow has been checked, false otherwise
 */
function deleteRowChecked(tableName, rownum)
{
  debug("In deleteRowChecked, tableName="+tableName+", rownum="+rownum);

  var deleteRowItem = dataForm[tableName+":deleteRow:"+rownum];

  if (  deleteRowItem
     && deleteRowItem.checked
     )
  {
    return true;
  }
  else
  {
    // return false for when there is no item deleteRowItem or when it is not checked
    return false;
  }
} // deleteRowChecked

/**
 * Validates form with name formName. 
 * Currently, this entails checking all tables in the form. Make
 * sure that you have registered the tables by calling addTableToBeValidated
 * for each table in the form. 
 */
function validateForm(formName)
{
  if (isFormValidationDisabled())
  {
    debug("form validation is disabled, do NOT perform checkForChanges ");
    enableFormValidation();
    return;
  }

  for (var i = 0; i < tablesToBeValidated.length; i ++)
  {
    // Only check the tables in form formName
    if (tablesToBeValidated[i][0] == formName)
    {
      // Check whether the form and the table exist before
      // validating it. It might be not the case when autoQuery
      // is turned off and no search criteria have been entered yet. 
      var dataForm = document.forms[formName];    
      if (dataForm && dataForm[tablesToBeValidated[i][1]+":length"])
      {
        var b = checkForChanges(formName, tablesToBeValidated[i][1]);
        if (!b) return false;
      }
    }
  }
  return true;
}

/**
 * This function should never be called directly anymore. Use validateForm
 * instead.
 *
 * checkForChanges is used when a table is submitted. The table should contain
 * a field 'hasChanges' for each row. This function populates those fields. If
 * a row has not changed then the value is 'N', if it has changes the value is
 * 'Y'. It also calls checkRequiredRowItems() to validate if there are required
 * items without a value.
 * note: UIX
 * @param formName: the name of the form
 * @param tableName: the name of the table to check for changes
 * @return: false when an error occured regarding a required item, true otherwise
 */
function checkForChanges(formName, tableName)
{
  debug("in checkForChanges for table "+tableName);
  // Set dataForm variable so that we don't need to supply the form name
  // to all helper functions of this method.
  dataForm       = document.forms[formName];
  // divide by one causes length to be a number
  var rowcount   = dataForm[tableName+":length"].value / 1;
  // Create array of all table items.
  var tableItems = createTableItemNamesArray(formName, tableName);
  var changeMade = false;
  var initialStatus;
  var item;

  // Loop through all the rows, and determine the record status
  for (var i = 0 ; i < rowcount; i++)
  {
    debug("determining record status for row "+i);
    
    changeMade    = false;

    if (  hasInsertStatus(tableName,i)
       && (  isEmptyDisplayLine(formName, tableItems, i)
          || deleteRowChecked(tableName, i)
          )
       )
    {
      // This record should be deleted (i.e. not submitted, i.e. no changes)
      changeMade = false;
    }
    else
    {
      // This record needs to be saved, even if the user does not manually
      // change any fields
      changeMade = true;
    }

    if (changeMade)
    {
      changeMade = false;
      // Loop through all items in the row to see if one was changed
      for (var j = 0 ; j < tableItems.length && !changeMade; j++)
      {
        // set hasChanges to 'Y' for row as soon as a changed item has been found
        item = dataForm[tableItems[j]+":"+i];
        // non-updateble items are rendered as text, so it should be checked if the item
        // exists before calling userHasChanged
        if (  item
           && userHasChanged(item)
           )
        {
          debug("Field "+tableItems[j]+" on row "+i+" was changed");
          changeMade = true;
        }
      }
    }

    // If a change was made to this row (and it does not have the delete checkbox checked)
    // check if the required fields for this row are filled in. If not, abort the loop
    // and return false;
    if (  changeMade
       && !deleteRowChecked(tableName, i)
       && !checkRequiredRowItems(formName, tableName, i)
       )
    {
      debug("checkForChanges returns false for table "+tableName);
      return false;
    }
  }

  debug("checkForChanges returns true for table "+tableName);
  return true;
} // checkForChanges


/**
 * getItemType returns the type of an item. If it is a radio group with more than 1 option,
 * the item is itself an array of radio group items.
 * note: UIX & JSP
 * @param item: the item for which to return the type
 * @return: the type of the item
 */
function getItemType(item)
{
  var type = null;

  if (!item)
  {
    alert("undefined item >"+item+"< in getItemType");
  }
  else
  {
    if (  !item.type
       && !isString(item)
       && item[0].type == "radio"
       ) // the item is an array of radio items
    {
      type = "radiogroup";
    }
    else if (!item.type && item[0] && item[0].type)
    {
      // Apparently item is an array. Just take the type of the first item
      type = item[0].type;
    }    
    else if (!item.type)
    {
      alert("unrecognized item >"+item+"< in getItemType");
    }
    else
    {
      type = item.type;
    }
  }

  return type;
} // getItemType


/**
 * isEmptyItem checks if an item is null or has an empty value
 * note: UIX & JSP
 * @param item: the item to check
 * @return: true when the item is null or has an empty value
 * (leading and trailing spaces are stripped before checking)
 */
function isEmptyItem(item)
{
  var type;

  if (!item)
  {
    alert("undefined item >"+item+"< in isEmptyItem");
    return null;
  }
  else
  {
    // getItemType will discover when item is an invalid parameter
    type = getItemType(item);

    // an isolated radio button should be ignored, i.e. return false
    if (type == "radio")
    {
      return false;
    }
    // a checkbox should be ignored, i.e. return false, as checked/unchecked are equivalent to
    // something like Y/N
    else if (type == "checkbox")
    {
      return false;
    }
    // the value of a radiogroup depends on one of the items having the property checked
    else if (type == "radiogroup")
    {
      var isEmpty = true;
      for (var i=0; i < item.length && isEmpty; i++)
      {
        isEmpty = !item[i].checked;
      }
      return isEmpty;
    }
    // the value of poplists need to be retrieved in a different way
    else if (type == "select-one" || type == "select-multiple")
    // the value of poplists is depends on the selectedIndex
    {
      // strip all leading and trailing spaces (in Netscape hidden items contain spaces)
      if (item.selectedIndex==-1)
      {  
        return '';
      }      
      return (stripSpaces(item.options[item.selectedIndex].value).length == 0);
    }
    // in other cases (password, text, textarea) the value depends on the property value
    else if (item.value)
    {
      // strip all leading and trailing spaces (in Netscape hidden items contain spaces)
      return (stripSpaces(item.value).length == 0);
    }
    else // when item has no value, it is considered to be empty
    {
      return true;
    }
  }
} // isEmptyItem


/**
 * resetValues loops through the global array 'reset' and resets all the fields
 * that correspond to the array items.
 * note: UIX & JSP
 * @param formName: the name of the form
 */
function resetValues(formName)
{
  var form = document.forms[formName];
  var item;
  var itemType;
  if (reset)
  {
    for (var i = 0 ; i < reset.length ; i++)
    {
      item = form[reset[i]];
      if (item)
      {
        itemType = getItemType(item);
        if (  itemType != "radio"
           && itemType != "radiogroup"
           && itemType != "checkbox"
           && itemType != "select-one"
           )
        {
          item.value = "";
        }
      }
    }
  }
} // resetValues


/**
 * resetFormValues loops through all forms, then through all form items of type 'hidden',
 * and if the hidden item did not have a default value, it is reset. The effect of this
 * is therefore similar to reloading a page, but only for hidden fields. This procedure
 * is called by default from the onLoad event of each UIX page, to make sure that (navigation)
 * hidden fields are cleared when the user performs a navigation, uses the backbutton, and
 * performs another navigation. On iExploder the hidden fields are not cleared automatically.
 * on Netscape they are.
 * note: UIX & JSP
 *       this function does not work properly for Netscape 7.0.2 as item.defaultValue for
 *       hidden items will be the same as the changed value
 */
function resetFormValues()
{
  var form;
  var item;
  for (var f = 0 ; f < document.forms.length ; f++)
  {
    form = document.forms[f];
    for (var i = 0 ; i < form.elements.length ; i++)
    {
      item = form.elements[i];
      if (  item.type == "hidden"
         && item.defaultValue == ""
         )
      {
        item.value = "";
      }
    }
  }
} // resetFormValues



/**
 * isString checks if the argument is a string
 * note: UIX & JSP
 * @param 'argument': the argument to check
 * @return: true if the 1st argument is a string, false otherwise
 */
function isString()
{
  if (arguments[0] == null)
  {
    return false;
  }
  else if (typeof arguments[0] == 'string')
  {
    return true;
  }
  else if (  (typeof arguments[0] == 'object')
          && (arguments[0].constructor)
          )
  {
    var criterion = arguments[0].constructor.toString().match(/string/i);
    return (criterion != null);
  }
  else
  {
    return false;
  }
}


/**
 * stripSpaces removes leading and trailing spaces from a string
 * note: UIX & JSP
 * @param aString: the string to strip
 * @return: the stripped string
 */
function stripSpaces(aString)
{
  if (!isString(aString))
  {
    alert("undefined string >"+aString+"< in stripSpaces");
    return null;
  }
  else
  {
    // strip leading spaces
    while (aString.substring(0,1) == " ")
    {
      aString = aString.substring(1);
    }
    // strip trailing spaces
    while (aString.substring(aString.length-1,aString.length) == " ")
    {
      aString = aString.substring(0,aString.length-1);
    }

    return aString
  }
} // stripSpaces


/**
 * isNumber checks if a string (after stripping leading and trailing spaces) represents
 * a numeric value. Empty strings are not considered to be a number.
 * This validator is more strict than the Marlin decimal validator, because it does not
 * accept values like "1a2".
 * Remark: values like "1e3" are accepted, because its interpreted as 1000 (one raised
 * to the power of three)
 * note: UIX & JSP
 * @param aString: the string to check
 * @return: true when aString represents a number, false otherwise
 */
function isNumber(aString)
{
  if (!isString(aString))
  {
    alert("undefined string >"+aString+"< in isNumber");
    return null;
  }
  else
  {
    // empty strings are not numbers
    if (stripSpaces(aString).length == 0)
    {
      return false;
    }

    // if aString evaluates to number zero, return true
    if ((aString*aString) == 0)
    {
      return true;
    }

    // in all other cases, return true if item divided by itself it returns 1
    return (aString/aString == 1);
  }
} // isNumber


/**
 * ignoreField checks if an item should be ignored when checking for changed items.
 * Items to be ignored are expected to start with an underscore (_) or be in the array
 * ignoreChangedFields
 * note: UIX & JSP
 * @param itemName: the item to be checked
 * @return: true when the item should be ignored (when the name start with an
 * underscore or when it is in the array ignoreChangedFields), false otherwise
 */
function ignoreField(itemName)
{
  debug("In ignoreField with itemName: "+itemName);

  itemNameStripped = stripRownumFromName(itemName);

  if (itemNameStripped.substring(0,1) == "_")
  {
    return true;
  }

  if (ignoreChangedFields)
  {
    for (var j = 0 ; j < ignoreChangedFields.length ; j++)
    {
      if (ignoreChangedFields[j] == itemNameStripped)
      {
        return true;
      }
    }
  }
  // now check whether the unstripped item name should be ignored 
  if (ignoreChangedFields)
  {
    for (var j = 0 ; j < ignoreChangedFields.length ; j++)
    {
      if (ignoreChangedFields[j] == itemName)
      {
        return true;
      }
    }
  }

  // return false in all other cases
  return false;

} // ignoreField


/**
 * hasChanges loops over all forms in the document and checks if one or more of the form items
 * have been changed by the user. If there are (unsumitted) changes, the user is asked
 * whether he wants to leave the page, or return to the page and save the changes. By calling
 * ignoreField, some of the items are ignored by hasChanges().
 * note: UIX & JSP
 * @return: true when there are unsubmitted changes (that might be because of an error on
 * the page), false otherwise
 */
function hasChanges()
{
  for (var i = 0 ; i < document.forms.length ; i++)
  {
    var form = document.forms[i];

    debug("checking form >"+form.name+"< for changes");

    // return true when there are errors on the page
    if (form.elements["hasChanges"])
    {
      if (stripSpaces(form.elements["hasChanges"].value) != "" )
      {
        debug("hasChanges returns true because hasChanges has a value");
        return true;
      }
      // else it concerns a select page
    }

    // return true when one of the items in any form have been changed by the user
    for (var k = 0 ; k < form.elements.length ; k++)
    {
      var item = form.elements[k];
      // fix for error in StripRowNum on UIX table pages in JDev 10.1.2
      // a new HTML element without a type is present on the page (for single selection UIX element)
      if (!item.type)
      {
        continue;
      }

      if (  !ignoreField(item.name)
         && userHasChanged(item)
         )
      {
        debug("hasChanges returns true because item >"+item.name+"< has changed");
        return true;
      }
    }
  }

  // in any other case, return false
  debug("hasChanges returns false");
  return false;
} // hasChanges


/**
 * confirmChanges checks if one of the form items has changed by calling hasChanges
 * If so, it asks the user to confirm if he wants to leave the page without saving,
 * or return to the same page. After the user has the opportunity to save the changes.
 * note: UIX & JSP
 * @return; true when there are no changes or when the user choose to leave the page
 * whithout saving them, false otherwise.
 */
function confirmChanges()
{
  var anyFormChanged = hasChanges();

  var confirmed;

  if (anyFormChanged)
  {
    confirmed = confirm(getMessage("hasChanged"));
  }

  if (!anyFormChanged || confirmed)
  {
    return true;
  }
  else
  {
    return false;
  }
} // confirmChanges



/**
 * DESCRIPTION TO BE PROVIDED
 * note: UIX
 */
function editorPopup(destination, formName, itemName, maxLength)
{
  destination += (destination.indexOf("?") == -1)
                 ? "?"
                 : "&";
  if (maxLength != null)
  {
    destination += "maxLength=" + maxLength;
  }
  top.editorFieldValue=document.forms[formName][itemName].value;
  popup(destination,formName,itemName,670,320);
} // editorPopup


/**
 * DESCRIPTION TO BE PROVIDED
 * note: UIX
 */
function popup(destination, formName, displayItemName, windowWidth, windowHeight, valueItemName)
{
  var displayField = document.forms[formName][displayItemName];

  var calWindow = openWindow( self
                            , destination
                            , "ModalWindow"
                            , {width:windowWidth, height:windowHeight}
                            , true
                            , "dialog"
                            , returnValue
                            );
  calWindow.displayField = displayField;
  if (valueItemName)
  {
    calWindow.valueField = document.forms[formName][valueItemName];
  }
} // popup


/**
 * returnValue copies the values set by de modal window and assigns them to
 * the form fields.
 * note: UIX
 * @param closingWindow: the modal window
 * @param event: not used
 */
function returnValue(closingWindow, event)
{
  var newDisplayValue = closingWindow.returnDisplayValue;
  var newValueValue   = closingWindow.returnValueValue;
  if (  !closingWindow.valueField
     && newDisplayValue != (void 0)
     )
  {
    // update the contents of the text field
    closingWindow.displayField.value = newValueValue;
  }
  else if (newDisplayValue != (void 0))
  {
    closingWindow.displayField.value = newDisplayValue;
  }

  if (  closingWindow.valueField
     && newValueValue != (void 0)
     )
  {
    // update the contents of the text field
    closingWindow.valueField.value = newValueValue;
  }
} // returnValue


/**
 * This function should be called from a button on a singleSelection
 * table. It check whether there is a selected row and, if so,
 * whether that row is not a new (and therefore not yet inserted) row.
 * To do this, the parameter checkItemName is passed. This should
 * be a field that is always null for new rows (no matter which fields
 * the end user already filled in, and that is never null for queried
 * rows. A perfect candidate would be the readOnly<PrimaryKey> formvalue.
 * To omit this check for new rows (for example if the table does not
 * have any new rows), call checkSelectedRow without the 3rd argument.
 * note: UIX
 */
function checkSelectedRow(formName, tableName, checkItemName)
{
  var form  = document.forms[formName];
  var field = form[tableName+":selected"];
  var j     = -1;

  if (field.length)
  {
    // than more than one radio group item is selectable
    for (var i = 0; i < field.length; i++)
    {
      debug("Checking if row "+i+" is checked");
      if (field[i].checked)
      {
        j = i;
      }
    }

    if (j == -1)
    {
      alert(getMessage("noRowsSelected"));
      return false;
    }
  }
  else
  {
    if (!field.checked)
    {
      alert(getMessage("noRowsSelected"));
      return false;
    }
  }

  if (checkItemName)
  {
    var checkItem = form[tableName+":"+checkItemName+":"+j];
    if (stripSpaces(checkItem.value) == "")
    {
      alert(getMessage("newRowSelected"));
      return false;
    }
  }

  var anyFormChanged = false;
  if (!ignoreChanges)
  {
    anyFormChanged = hasChanges();
  }

  var confirmed;
  if (  !anyFormChanged
     || confirm(getMessage("hasChanged"))
     )
  {
    return true;
  }
  else
  {
    return false;
  }
} // checkSelectedRow


/**
 * changeDisplayPoplist give a warning if displayPoplist has changed, while there are
 * unsaved changes in form. When the user confirms the contextChangedItem is to 'ok'
 * and the form is submitted. Otherwise the the displayPoplist is reset to it's orginal
 * value.
 * Explanation:
 * A display poplist contains values which influence the display of certain fields in the form,
 * for example Hours or Days. If the users gets the warning and chooses Cancel,
 * the display poplist is restored to the value it had when the screen was originally painted
 * (consistent with refreshing the screen whenever the display poplist is changed).
 * note: UIX & JSP (should move to utility library)
 * @param form: name of form
 * @param contextChangedItem: form field that must be set to 'ok' if context has changed,
 * so that jHeadstart knows which process to call next (typically a refresh of the data,
 * using the right display unit)
 * @param displayPoplist: field name of messageChoice field that contains allowable display
 * values (for example hours and days)
 * @return: false, when the user does not confirm the change
 */
function changeDisplayPoplist(form, contextChangedItem, displayPoplist)
{
  var changed   = hasChanges();
  var confirmed = true;
  if (  hasChanges()
     && !ignoreChanges
     )
  {
    confirmed = confirm(getMessage("hasChanged"));
  }

  if (confirmed)
  {
    contextChangedItem.value = "ok";
    form.submit();
  }
  else
  {
    /* go back to original value of displayPoplist */
    for (var i = 0; i < displayPoplist.options.length; i++)
    {
      if (displayPoplist.options[i].defaultSelected)
      {
        displayPoplist.selectedIndex = i;
      }
    }
    return false;
  }
} // changeDisplayPoplist


/**
 * DESCRIPTION TO BE PROVIDED
 * note: UIX
 */
function lovpagePopup(target, formName, displayItemName, valueItemName)
{
  destination = target + "&destination="+document.location;
  popup(destination,formName,displayItemName,800,500,valueItemName);
} // lovpagePopup


/**
 * replace replaces all occurences of a specific text in a string
 * note: UIX & JSP (should move to utility library)
 * @param string: the string for which text needs to be replaced
 * @param text: the text that needs to be replaced
 * @param by: the replacement text
 */
function replace(aString, text, by)
{
  if (  !isString(aString)
     || !isString(text)
     || !isString(by)
     )
  {
    alert("undefined aString >"+aString+", "+text+", "+by+"< in replace");
    return null;
  }
  else
  {
    var strLength = aString.length;
    var txtLength = text.length;

    // return the original string when either the string or the text to be
    // replaced have length 0
    if (  (strLength == 0)
       || (txtLength == 0)
       )
    {
      return aString;
    }

    var i = aString.indexOf(text);
    if (  (!i)
       && (text != aString.substring(0,txtLength))
       )
    {
      return aString;
    }

    if (i == -1)
    {
      return aString;
    }

    var newstr = aString.substring(0,i) + by;
    if ((i + txtLength) < strLength)
    {
      newstr += replace(aString.substring(i+txtLength,strLength),text,by);
    }

    return newstr;
  }
} // replace



/**
 * submitJspForm submits an html form. Before submitting the form, submitJspForm
 * may check whether there are changes. Whether the function checks for changes, 
 * depends in the first place on the parameter isCheckForChangesButton. This parameter
 * is 'true' when the form should be checked for changes. 
 * All the buttons will have isCheckForChangesButton is 'true' except the buttons 
 * that have as function to update the database using the information in the form, e.g. save 
 * and delete buttons. 
 * Second of all, whether the function checks for changes, depends on the global variable
 * ignoreChanges. This variable will have been set to 'true', if the form never has to be checked,
 * e.g. a find page form. 
 * If the function will check for changes and there are changes, the user will be asked if he
 * really wants the form to be submitted. This is a good thing, because submitting the form will
 * meaning losing your changes.
 * Information about the button pressed should be made part of the form before the form can 
 * can be submitted, such that Struts or MVC knows what to do. The name and value parameters
 * contain the name and value of the button pressed. submitJspForm will fill the hidden fields
 * targetCondition (name) and targetConditionValue (value). These hidden fields are, and have to be, part
 * of every form that can be submitted. 
 * Before submitting the form, submitJspForm will close all opened windows, such as calendar, lov or
 * editor windows. 
 *
 * note: JSP
 * @param formName: The name of the form to be submitted
 * @param name: The name of the button pressed
 * @param value: The value of the button pressed
 * @param isCheckForChangesButton: 'true' if the form should be checked for changes before submitting
 *                                  the form, else 'false'
 */
function submitJspForm(formName, name, value, isCheckForChangesButton)
{
  var form    = document.forms[formName];

  var changed = false;
  if (!ignoreChanges && isCheckForChangesButton)
  {
    changed = hasChanges();
  }

  var confirmed;
  if (changed)
  {
    confirmed = confirm(getMessage("hasChanged"));
  }

  if (!changed || confirmed)
  {
    closeWindows();
    form.targetCondition.value = name;
    form.targetConditionValue.value = value;
    form.submit();
  }
} // submitJspForm


/**
 * submitJspAdfForm submits an html form. See submitJspForm for more info.
 * The difference with submitJspForm is that the form fields that are populated
 * are specified in the keys argument, and the values used to populate these fields
 * are specified in the values argument.
 * ADF expects parameter names like event_Commit, so we cannot use targetCondition
 * and targetConditionValue
 *
 * note: JSP
 * @param formName: The name of the form to be submitted
 * @param keys: The name of the button pressed
 * @param values: The value of the button pressed
 * @param isCheckForChangesButton: 'true' if the form should be checked for changes before submitting
 *                                  the form, else 'false'
 */
 function submitJspAdfForm(formName, keys, values, isCheckForChangesButton)
 {
  var form    = document.forms[formName];

  var changed = false;
  if (!ignoreChanges && isCheckForChangesButton)
  {
    changed = hasChanges();
  }

  var confirmed;
  if (changed)
  {
    confirmed = confirm(getMessage("hasChanged"));
  }

  if (!changed || confirmed)
  {
    closeWindows();
    for (var i = 0 ; i < keys.length ; i++)
    {
      form[keys[i]].value = values[i];
    }

    form.submit();
  }
} // submitJspAdfForm


/**
 * closeWindows closes the lovWindow, calendarWindow and editorWindow if they are still open.  
 * note: JSP
 */
function closeWindows()
{
  if (lovWindow) lovWindow.close();
  if (calendarWindow) calendarWindow.close();
  if (editorWindow) editorWindow.close();
} // closeWindows

/**
 * Opens an editor window. 
 *
 * note: JSP
 * @param destination The url pointing to the editor
 * @param itemName The field in which the result of the editor will be stored
 * @param maxLength The maximum amount of characters allowed in the editor. 
 */
function jspEditorPopup(destination, itemName, maxLength)
{
  editorMaxlength = maxLength;

  editorField = getFormElement(itemName);
  editorValue = editorField.value;

  editorWindow = window.open
  (
    destination
  , "editor"
  , "toolbar=no,location=no,directories=no,status=no,menubar=no,scrollbars=no," +
    "resizable=no,dependent=yes, width=350, height=190, screenX=200, screenY=300"
  );

} // editorPopup


/**
 * Opens a lov popup. Furthermore this function initializes the variables
 * lovField and baseValueAttribute. In these fields, the result of the lov popup 
 * is stored.
 * 
 * @param lovName The name of the lov
 * @param itemName The name of the field where the display result should be put
 * @param baseValueAttribute The name of the field where the base value result should be put
 * note: JSP
 */
function jspLovPopup(lovName, itemName, baseValueAttributeName)
{
  // Instead of having the form as a parameter and looking at the right form 
  // straight away, we just loop through all of the loops and see if we can find
  // field with names itemName and baseValueAttributeName.
  // This way the jsp code and xslt code can be made a bit easier. The drawback
  // is of course that if a jsp contains multiple forms containing fields used with lovs
  // with the same name, then things will go wrong. This will probably never happen, certainly
  // not with generation.

  lovField = getFormElement(itemName);
  baseValueAttribute = getFormElement(baseValueAttributeName);

  lovWindow = window.open
  (
    "?targetGroup=Lov" + lovName
  , "lov"
  , "toolbar=no,location=no,directories=no,status=no,menubar=no,scrollbars=no," +
    "resizable=yes,dependent=yes, width=450, height=500"
  );

} // lovPopup


/**
 * Updates a page when an option in a lov popup has been selected. This function
 * requires that the variable lovField has been initialized with the field to store
 * the display value (e.g. "King") in. If the variable baseValueAttribute has been 
 * initialized as well, then the base value of the option, e.g. the foreign
 * key value 10, will be stored in this field. 
 *
 * note: JSP
 * @param displayValue The display value of the selected option
 * @param valueValue The base value of the selected option
 */
function chooseLovValue(displayValue, valueValue)
{
  lovField.value = displayValue;
  if (baseValueAttribute)
  {
    baseValueAttribute.value = valueValue;
  }
} // chooseLovValue


/**
 * Opens the calendar window. Before opening the window it will call the functions
 * setDateField and setDateFormat in calendar.js such that the calendar knows
 * where to put the result of the calendar window (setDateField) and which date format
 * to use (setDateFormat).
 *
 * note: JSP
 * @param calendarPath The relative URI to locate the calendar html page
 * @param itemName The element where the result of the calendarWindow
 *                 should be put
 * @param format   The date format
 */
function calendarPopup(calendarPath, itemName, format)
{
  setDateField(getFormElement(itemName)); 
  setDateFormat(format);

  calendarWindow = window.open
  (
    calendarPath
  , "calendar"
  , "toolbar=no,location=no,directories=no,status=no,menubar=no,scrollbars=no,resizable=no,dependent=yes, width=210, height=230, screenX=200, screenY=300"
  );
} // lovPopup

/**
 * Loops through all the forms and look for the first occurence of an element with name
 * elementName.
 *
 * note: JSP
 * @param elementName The name of the element to look for
 * @result The first occurence of the element with name elementName. null, if no
 *         element with name elementName could be found
 */
function getFormElement(elementName)
{
  var result = null;

  for (var i = 0 ; i < document.forms.length ; i++)
  {
    var searchForm = document.forms[i];
    if (searchForm.elements[elementName])
    {
      result = searchForm.elements[elementName];
    }
  }

  return result;
}

/**
 * Loops through all the forms and looks for the first occurence of an element with name
 * elementName. The function returns the value of this element. 
 *
 * note: JSP
 * @param elementName The name of the element to look for
 * @result The value of the first occurence of the element with name elementName. null, if no
 *         element with name elementName could be found
 */
function getFormElementValue(elementName)
{
  var result = null;
  var formElement = getFormElement(elementName);
  if (formElement)
  {
    result = formElement.value;
  } 
  return result;
}



/****************** NIEUWE FUNCTIES VOOR ADF! *********/

function doNavigate(actionUrl, event, rowKeyStr, resetBreadcrumbs)
{
  // reset hidden fields for IE
  resetValues(routerFormName);
 
  var form    = getRouterForm();
 
  form.action=actionUrl;
 
  var changed = false;
  if (!ignoreChanges)
  {
    changed = hasChanges();
  }
 
  var confirmed;
  if (changed)
  {
    confirmed = confirm(getMessage("hasChanged"));
  }
 
  if (!changed || confirmed)
  {
    if (event!=null)
    {
      form.event.value = event;
    }
    if (rowKeyStr!=null)
    {
      form.rowKeyStr.value = rowKeyStr;
    }
    // if resetBreadcrumbs was not passed in or is true, set the field
    // which resets the breadcrumb stack.
    if (resetBreadcrumbs == null || resetBreadcrumbs == true)
    {
      form.resetBreadcrumbs.value='true';
    }
    form.submit()
  }
} // 

// doEvent submit routerForm
function doEvent(event, eventValue, checkForChanges, rowKeyStr ,doRollback)
{

  // reset hidden fields for IE
  resetValues(routerFormName);

  var form    = getRouterForm();

  if (form.action == "" || form.action == null)
  {
    form.action = document.forms[dataFormName].action;
  }

  if(checkForChanges == null)
  {
    checkForChanges = true;
  }

  var changed = false;
  var confirmed;
  if (!ignoreChanges && checkForChanges)
  {
    changed = hasChanges();

    if (changed)
    {
      confirmed = confirm(getMessage("hasChanged"));
    }
  }

  if (!changed || confirmed)
  {
    form.event.value = event;
    form.rowKeyStr.value = rowKeyStr;
    form.eventValue.value = eventValue;
    if(doRollback == null)
    {
      doRollback = 'true';
    }
    form.event_Rollback.value = doRollback;
    form.submit()
  }
} // 

// doEvent submit routerForm
function doDataEvent(event, eventValue, checkForChanges)
{
  // reset hidden fields for IE
  // S. Davelaar: commented out: this causes Quick Search to fail
  // which relies ion hidden fields passed in.
   resetValues(dataFormName);

   var form    = getDataForm();

  if(checkForChanges == null)
  {
    checkForChanges = false;
  }

  var changed = false;
  if (!ignoreChanges && checkForChanges)
  {
    changed = hasChanges();
  }

  var confirmed;
  if (changed)
  {
    confirmed = confirm(getMessage("hasChanged"));
  }

  if (!changed || confirmed)
  {
    form.event.value = event;
    form.eventValue.value = eventValue;
    form.submit()
  }
} // 

function getRouterForm()
{
  return document.forms[routerFormName];
}

function getDataForm()
{
  return document.forms[dataFormName];
}

function AdfJspLovPopup(lovName, itemName, baseValueAttributeName)
{
  // Instead of having the form as a parameter and looking at the right form 
  // straight away, we just loop through all of the loops and see if we can find
  // field with names itemName and baseValueAttributeName.
  // This way the jsp code and xslt code can be made a bit easier. The drawback
  // is of course that if a jsp contains multiple forms containing fields used with lovs
  // with the same name, then things will go wrong. This will probably never happen, certainly
  // not with generation.

  lovField = getFormElement(itemName);
  baseValueAttribute = getFormElement(baseValueAttributeName);

  lovWindow = window.open
  (
    "Lov" + lovName + ".do?pageTimeStamp=ignore"
  , "lov"
  , "toolbar=no,location=no,directories=no,status=yes,menubar=no,scrollbars=yes," +
    "resizable=yes,dependent=yes, width=550, height=500"
  );

} // lovPopup

         function setClassOnElement( elementId, className)
         {
            el = document.getElementById(elementId);
            agt = navigator.userAgent.toLowerCase();
            if (agt.indexOf('mozilla')!=-1 && agt.indexOf('spoofer')==-1 && agt.indexOf('compatible') == -1)
              // mozilla
              el.setAttribute("class", className);
            else
              el.setAttribute("className", className);
         }


function addToIgnoreChangedFields(fields) 
{
  curlength = ignoreChangedFields.length;
  for (i = 0; i<fields.length; i++)
    ignoreChangedFields[curlength + i] = fields[i];
}

/**
 * Append a new array of prompts to array "itemScreenPrompts"
 */
function addItemScreenPrompts(prompts) 
{
  curlength = itemScreenPrompts.length;
  for (i = 0; i<prompts.length; i++)
    itemScreenPrompts[curlength + i] = prompts[i];
}

/**
 * Append a new array of row items to array "requiredRowItems"
 */
function addRequiredRowItems(items) 
{
  curlength = requiredRowItems.length;
  for (i = 0; i<items.length; i++)
    requiredRowItems[curlength + i] = items[i];
}

function addTableToBeValidated(form, tableName)
{
  tuple = new Array();
  tuple[0] = form;
  tuple[1] = tableName;
  tablesToBeValidated[tablesToBeValidated.length] = tuple;
}

/**
 * Function that can be used with UIX LOV.
 * Through Partial Page rendering, UIX supports to copy multiple values
 * from the selected Lov row, back to the calling form. However, apart from
 * the messageLovInput field itself, the other "target" items must be readOnly
 * in other for PPR to work. If we want to copy back values to, for example,
 * hidden fields (a common scnenario when the LOV is attached to a descriptor field)
 * we can use the Javascript callback mechanism that is supported by the messageLovInput
 * element. To use this callback in combination with this javascript function, you
 * must do the following:
 * - set onLovSelect property of the messageLovInput to "LovSelectCallBack"
 * - import the UIX tableProxy.js library in your page
 * - In the Lov page, add a hidden field named 'tableName' that holds the name of the lov table
 * - In the Lov page, add a hidden field named 'valueField' that holds the name of field that 
 *   contains the value that must be copied back to the calling form
 * - In the Lov page, add a hidden field named 'targetValueField' that holds the name of field 
 *   in the calling form that must be updated.
 *
 * note: UIX only
 *
 * To DO: Add support for more than one value to be copied back.
 *
 */
function LovSelectCallBack(lovWin, field, event, mainWin)
{
  // prevent custom javascript validation of required fields in table (see also checkForChanges)
  disableFormValidation();  

  var form = lovWin.document.forms[0];
  if (form.valueField)
  {
    var tableName = form.tableName.value;
    var valueField = form.valueField.value;
    var targetValueField = form.targetValueField.value;

    var proxy = new TableProxy(tableName,form.name, lovWin);
    var selRow = proxy.getSelectedRow();
    var selVal = proxy.getFormElement(valueField, selRow).value;
    mainWin.document.forms[0].elements[targetValueField].value = selVal;
  }
  return true;
}

/**
 * Call this function when your input control creates a partial page requests
 * that submits the form, and Jheadstart should NOT do custom javascript
 * validation like checking required items in a table
 */
function disableFormValidation()
{
  if (getDataForm().validateForm)
  {
    getDataForm().validateForm.value='false';
  }
}  

/**
 * Call this function to enable Jheadstart ustom javascript
 * validation like checking required items in a table
 */
function enableFormValidation()
{
  if (getDataForm().validateForm)
  {
    getDataForm().validateForm.value = "true";
  }
}  

function isFormValidationDisabled()
{
  if (getDataForm().validateForm)
  {
    return getDataForm().validateForm.value=='false';
  }
  else
  {
    return true;
  }
}  

function alertForChanges()
{
  var changed = hasChanges();
  var confirmed;
  if (changed)
  {
     confirmed = confirm(getMessage("hasChanged"));
  }

  if (!changed || confirmed)
  {
    return true;
  }
  else
  {
    return false;
  }
}

function selectTreeNode(action, selection, iterPath, keyPath)
{
    var form    = document.forms['treeRouterForm'];
    form.action = action;
 
    form.selection.value = selection;
    form.iterPath.value = iterPath;
    form.keyPath.value = keyPath;
    form.submit();
} 

/**
 * Clears all the advanced search find fields in form with name formName.
 * It does this by setting the value null of elements in the form that 
 * have a name that satisfies the following criteria
 * - It starts with Find
 * - It doesn't include a :, which probably means that the element is part of
 *   a uix table, which most likely means that it shouldn't be reset
 * - It doesn't include a [, which probably means that the element is part of
 *   a jsp table, which most likely means that it shouldn't be reset
 */
function clearAdvancedSearch(formName)
{
  var form = document.forms[formName];
  var formElementsCount  = form.elements.length / 1;  
  for (var i = 0; i < formElementsCount; i ++)
  {
    var item = form.elements[i];
    var name = item.name;
    if (name && name.substring(0, 4) == 'Find' && name.indexOf(":") == -1 && name.indexOf("[") == -1 && item.value)
    {
      item.value = '';
    }
  }
}

